use crate::TallyState;
use actix::prelude::*;
use futures::sync::mpsc::{Receiver, Sender};

#[derive(Message)]
pub struct Message(pub String);

#[derive(Message)]
pub struct Connect {
    pub addr: Recipient<Message>,
}

#[derive(Message)]
pub struct NewState(Result<TallyState, ()>);

// Basically just convenience so you don't have to construct a NewState with an Ok all the time
impl From<TallyState> for NewState {
    fn from(state: TallyState) -> Self {
        NewState(Ok(state))
    }
}

pub struct WSServer {
    state_tx_tx: std::sync::mpsc::Sender<Sender<NewState>>,
}

impl StreamHandler<NewState, ()> for WSServer {
    fn handle(&mut self, item: NewState, ctx: &mut Context<WSServer>) {
        log::info!("Weobsockety! {}", item.0.unwrap());
    }
}

impl Actor for WSServer {
    type Context = Context<Self>;

    fn started(&mut self, ctx: &mut Self::Context) {
        log::info!("Websocket started");
        let (tx, rx) = futures::sync::mpsc::channel::<NewState>(10);
        Self::add_stream(rx, ctx);
        self.state_tx_tx.send(tx).unwrap();
    }
}

impl WSServer {
    pub fn new(state_tx_tx: std::sync::mpsc::Sender<Sender<NewState>>) -> WSServer {
        WSServer { state_tx_tx }
    }
}
